@@ -126,6 +126,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u'p',u'(.*)pi',u'siipi'),
 			(u'k>',u'(.*i)ki',u'piki'),
 			(u'k>',u'(.*)ki',u'kaikki')]),
+	(u'suksi', u'-', [(None,u'(.*u)ksi',u'suksi')]),
 	(u'veli', u'-', [(None,u'(.*el)i',u'veli')]),
 	(u'nalle', u'sw', [(None,u'(.*Ce)',u'nalle'),
 			 (None,u'(.*Cé)',u'nalle'),
@@ -147,6 +148,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u'k>',u'(.*AA)kA',u'raaka'),
 			(u'k>',u'(.*)kA',u'haka')]),
 	(u'nahka', u'-', [(None,u'(.*)kA',u'nahka')]),
+	(u'jumala', u'-', [(None,u'(.*l)A',u'jumala')]),
 	(u'koira', u'sw',   [(None,u'(.*)A',u'koira'),
 			(u'tt',u'(.*t)tA',u'kenttä'),
 			(u'nk',u'(.*n)kA',u'honka'),
@@ -216,24 +218,31 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 	(u'sisar', u'ws', [(None,u'(.*CVC)',u'sisar'),
 		         (u't',u'(.*t)Ar',u'tytär'),
 		         (u'>k',u'(.*i)en',u'ien')]),
+	(u'hapan', u'-', [(None,u'(.*p)An',u'hapan')]),
 	(u'uistin', u'ws', [(None,u'(.*[iaä])n',u'uistin'),
 	                    (u'nn',u'(.*n)nin',u'vaimennin'),
 		          (u'll',u'(.*l)lin',u'sivellin'),
 		          (u'rr',u'(.*r)rin',u'kiharrin'),
+		          (u'rr',u'(.*r)rOin',u'kerroin'),
 		          (u'd',u'(.*)din',u'kaadin'),
 		          (u'v',u'(.*)vin',u'kaavin'),
 			  (u't',u'(.*t)in',u'suodatin'),
 			  (u'k',u'(.*k)in',u'puin'),
 		          (u'j>k',u'(.*l)jin',u'poljin'),
 		          (u'>k',u'(.*)in',u'puin')]),
+	(u'laidun', u'-', [(None,u'(.*)dUn',u'laidun')]),
 	(u'onneton', u'ws', [(None,u'(.*t)On',u'alaston'),
 	                     (u't',u'(.*t)On',u'onneton')]),
+	(u'lämmin', u'-', [(None,u'(.*m)min',u'lämmin')]),
+	(u'vasen', u'-', [(None,u'(.*e)n',u'vasen')]),
 	(u'sisin', u'', [(None,u'(.*)in',u'pahin')]),
 	(u'nainen', u'-', [(None,u'(.*)nen',u'nainen')]),
 	(u'vastaus', u'-', [(None,u'(.*V)s',u'vastaus')]),
 	(u'kalleus', u'-', [(None,u'(.*VU)s',u'kalleus'),
 	                    (None,u'(.*vU)s',u'kalleus')]),
 	(u'kaunis', u'-', [(None,u'(.*C)is',u'kaunis')]),
+	(u'autuas', u'-', [(None,u'(.*U)As',u'autuas')]),
+	(u'laupias', u'-', [(None,u'(.*p)iAs',u'laupias')]),
 	(u'vieras', u'ws', [(None,u'(.*[lmr]i[aä])s',u'antelias'),
 		            (None,u'(.*il[aä])s',u'antelias'),
 		            (None,u'(.*A)s',u'vieras'),
@@ -275,9 +284,11 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 		        (u't',u'(.*Vt)e',u'vaate'),
 		        (u't',u'(.*lt)e',u'vaate'),
 		        (u't',u'(.*rt)e',u'vaate')]),
+	(u'alkeet', u'-', [(None,u'(.*ke)et',u'alkeet')]),
 	(u'tie', u'-', [(None,u'(.*t)ie',u'tie')]),
 	(u'lapsi', u'-', [(None,u'(.*)psi',u'lapsi')]),
 	(u'hapsi', u'-', [(None,u'(.*)psi',u'hapsi')]),
+	(u'loppu', u'-', [(None,u'(.*)',u'loppu')]),
 	(u'veitsi', u'-', [(None,u'(.*)tsi',u'veitsi')]),
 	# Verbs
 	(u'punoa', u'sw', [(None,u'(.*)A',u'punoa'),
@@ -346,6 +357,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u't',u'(.*)tAA',u'raataa')]),
 	(u'kaikaa', u'-', [(None,u'(.*C)AA',u'kapsaa')]),
 	(u'soutaa', u'sw', [(u't',u'(.*)tAA',u'soutaa')]),
+	(u'saartaa', u'-', [(None,u'(.*r)tAA',u'saartaa')]),
 	(u'laskea', u'sw', [(None,u'(.*C)eA',u'laskea'),
 	                    (u'nk',u'(.*n)keA',u'tunkea'),
 			(u't',u'(.*)teA',u'kutea'),
@@ -413,6 +425,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u'v',u'(.*)vetA',u'kaveta'),
 			(u'j>k',u'(.*)jetA',u'tarjeta'),
 			(u'>k',u'(.*)OtA',u'ulota'),
+			(u'>k',u'(.*)AtA',u'erata'),
 			(u'>k',u'(.*)etA',u'vaieta')]),
 	(u'haluta', u'ws', [(None,u'(.*C)itA',u'selvitä'),
 			(None,u'(.*gO)tA',u'bingota'),
