@@ -77,12 +77,28 @@ def get_prefix_jatko(word):
 
 def get_adverb_jatko(word):
 	flags = generate_lex_common.get_flags_from_group(word, u"inflection")
-	prefixJatko = u"loppu"
+	prefixJatko = u""
+	loppu = True
 	for flag in flags:
-		if flag in [u"liitesana"]:
+		if flag in [u"liitesana", u"omistusliite"]:
 			prefixJatko = prefixJatko + u", " + flag
+		elif flag == u"ulkopaikallissijat_yks":
+			prefixJatko = prefixJatko + u", ulkopaikallissija_llA"
+		elif flag == u"required":
+			loppu = False;
+	if loppu:
+		prefixJatko = prefixJatko + u", loppu"
+	if prefixJatko.startswith(u", "):
+		prefixJatko = prefixJatko[2:]
 	return prefixJatko
 
+def get_abbreviation_jatko(word, wordform):
+	flags = generate_lex_common.get_flags_from_group(word, u"inflection")
+	if wordform.endswith(u".") or u"none" in flags:
+		return u"loppu"
+	else:
+		return u"tavuviiva, kaksoispiste, loppu"
+
 def get_additional_attributes(word):
 	flags = generate_lex_common.get_flags_from_group(word, u"compounding")
 	result = u""
@@ -113,7 +129,7 @@ def handle_word(word):
 	
 	# Get the word classes
 	wordclasses = generate_lex_common.tValues(word.getElementsByTagName("classes")[0], "wclass")
-	if wordclasses[0] not in [u"interjection", u"prefix", u"abbreviation", u"adverb"] and voikko_infclass == None:
+	if wordclasses[0] not in [u"interjection", u"prefix", u"abbreviation", u"conjunction", u"adverb"] and voikko_infclass == None:
 		return
 	malaga_word_class = generate_lex_common.get_malaga_word_class(wordclasses)
 	if malaga_word_class == None: return
@@ -159,7 +175,7 @@ def handle_word(word):
 			sys.stderr.write(errorstr.encode(u"UTF-8"))
 			sys.exit(1)
 		if malaga_word_class == u"lyhenne":
-			jatko = u"tavuviiva, kaksoispiste, loppu"
+			jatko = get_abbreviation_jatko(word, altform)
 		elif malaga_word_class == u"seikkasana":
 			jatko = get_adverb_jatko(word)
 		if malaga_word_class == u"etuliite":
