@@ -35,17 +35,17 @@
 # spellings and spelling errors that do not exist in real life.
 
 # Compiling (in directory suomimalaga):
-# GENLEX_OPTS=--sukija make -e vvfst-sukija
+# make vvfst-sukija
 
 # Testing:
 # foma -e "read att all-sukija.att" -e "save stack sukija.fst" -e "quit"
 # date; cat ~/Lataukset/koesanat?.txt | flookup -i sukija.fst | gawk 'length($0) > 0' |sort >test.out; date
 # diff test.out ~/Lataukset/vvfst-sukija-testi.out | grep '<.*[+][?]' | less
 
-# Tiedostossa taivutuskaavat.lexc.in vanhat taivutukset on lisätty näihin taivutuksiin:
-# Hame, Kalleus, Kansi, Koira, Lohi, Lovi, Lumi, Meri, Nainen, Onneton, Paperi,
-# Pieni, Risti, Sisar, Tosi, Uistin, Veli.
-# 17.6.2013: Kalleus, Kansi, Lohi, Lumi, Meri, Pieni, Tosi.
+# Style- ja usage-lippujen arvot suoraan Joukahaisesta:
+# grep -A1 '<style>' ../vocabulary/joukahainen.xml|grep flag|sort -u|gawk '{printf "%s,", substr($1,7,length($1)-13)}'
+# grep -A1 '<usage>' ../vocabulary/joukahainen.xml|grep flag|sort -u|gawk '{printf "%s,", substr($1,7,length($1)-13)}'
+
 
 import codecs
 import getopt
@@ -78,6 +78,7 @@ def makeRe (wordClass, word):
 def replace (s, old, new):
     u = s.replace (old + u":", new + u":")
     u = u.replace (old + u" ", new + u" ")
+    u = u.replace (old + u"\t", new + u"\t")
     u = u.replace (old + u"@", new + u"@")
     return u
 
@@ -122,6 +123,8 @@ re_uutio = makeRe (u"Ln", u".Cuutio")
 re_uusio = makeRe (u"Ln", u".Cuusio")
 re_tio   = makeRe (u"Ln", u"[ik]tio") # Traditio, funktio.
 
+re_toninen = makeRe (u"Ll", u".toninen")
+
 re_nuolaista = re.compile (u"\\[Lt\\].* Nuolaista_");
 re_rangaista = re.compile (u"\\[Lt\\].* Rangaista_");
 
@@ -138,8 +141,6 @@ re_logia_x = re.compile (u"\\A\[Ln\]\[Xp\](genealogia|trilogia)\[X\]")
 re_uusio_x = re.compile (u"\\A\[Ln\]\[Xp\](diffuusio)\[X\]")
 re_tio_x   = re.compile (u"\\A\[Ln\]\[Xp\](aitio)\[X\]")
 
-Joukahainen_t = re.compile (u"\\AJoukahainen_t.*;")
-
 re_A = re.compile (u"[aou]")
 
 
@@ -168,10 +169,10 @@ spelling_pattern_list = [
   (re_loginen,  u"logi",   u"loogi"),
 
   (re_oitin, u"oit", u"ot"),  # Kirjoitin => kirjotin (esim. kirjo(i)ttimen).
-  (re_aatio, u"aatio", u"atsion", u"atsioon", u"NimisanaAutio_a", u"NimisanaPaperi_a"),
-  (re_uutio, u"uutio", u"utsion", u"utsioon", u"NimisanaAutio_a", u"NimisanaPaperi_a"),
-  (re_uusio, u"uusio", u"usion",  u"usioon",  u"NimisanaAutio_a", u"NimisanaPaperi_a", re_uusio_x),
-  (re_tio,   u"tio",   u"tsion",  u"tsioon",  u"NimisanaAutio_a", u"NimisanaPaperi_a", re_tio_x),
+  (re_aatio, u"aatio", u"atsio", u"atsion", u"atsioon", u"NimisanaAutio_a", u"NimisanaPaperi_a"),
+  (re_uutio, u"uutio", u"utsio", u"utsion", u"utsioon", u"NimisanaAutio_a", u"NimisanaPaperi_a"),
+  (re_uusio, u"uusio", u"usio",  u"usion",  u"usioon",  u"NimisanaAutio_a", u"NimisanaPaperi_a", re_uusio_x),
+  (re_tio,   u"tio",   u"tsio",  u"tsion",  u"tsioon",  u"NimisanaAutio_a", u"NimisanaPaperi_a", re_tio_x),
   (re_oittaa1, u"o",   u"ot",  u"Kirjoittaa", u"Alittaa"),
   (re_oittaa2, u"ö",   u"öt",  u"Kirjoittaa", u"Alittaa"),
   (re_oittaa1, u"oit", u"ot",  u"Alittaa",    u"Alittaa"),
@@ -180,6 +181,9 @@ spelling_pattern_list = [
   (re_ottaa2,  u"öt",  u"öit", u"Alittaa",    u"Alittaa"),
   (re_ottaa1,  u"o",   u"oi",  u"Ammottaa",   u"Ammottaa"),
   (re_ottaa2,  u"ö",   u"öi",  u"Ammottaa",   u"Ammottaa"),
+
+  (re_toninen, u"toni", u"tooni"),
+
   (re_nuolaista, u"Nuolaista_"),
   (re_rangaista, u"Rangaista_"),
 ]
@@ -233,12 +237,6 @@ def generate (lexicon, word_class, a):
     return p
 
 
-def write_word (p):
-    prefix = p[1][0:p[1].find (u"[X]")+3]
-    outfile.write (u"%s%s:%s Sukija%s ;\n" % (prefix, p[0][0:len(p[0])-1], p[0][0:len(p[0])-1], p[2]))
-
-
-
 def word_class (line):
     L = dict ([(u"[Ll]",  u"Laatusana"),
                (u"[Ln]",  u"Nimisana"),
@@ -281,10 +279,11 @@ def generate_from_pattern (line, pattern_list):
                 replace_and_write (line, x[1], x[2])
             elif (len(x) == 5) and (line.find (x[3]) >= 0):
                 replace_and_write (line.replace(x[3], x[4]), x[1], x[2])
-            elif (len(x) == 6) or (len(x) == 7 and not x[6].match(line)):
-                s = line.replace (x[4], x[5])
-                outfile.write (replace (s, x[1], x[2]))
+            elif (len(x) == 7) or (len(x) == 8 and not x[7].match(line)):
+                s = line.replace (x[5], x[6])
                 outfile.write (replace (s, x[1], x[3]))
+                outfile.write (replace (s, x[1], x[4]))
+                outfile.write (replace (line, x[1], x[2]))
 
 
 # Old spellings and common spelling errors of words
@@ -294,9 +293,11 @@ def generate_from_pattern (line, pattern_list):
 #
 word_list = [
     (u"aarteisto",        (u"aarteisto",     u"aartehisto")),
+    (u"aggressiivinen",   (u"aggressiivi",   u"akressiivi")),
     (u"aikainen",         (u"aikai",         u"aikahi")),
     (u"ainainen",         (u"ainai",         u"ainahi")),
     (u"aineisto",         (u"aineisto",      u"ainehisto")),
+    (u"aksiomi",          (u"aksiom",        u"aksioom")),
     (u"alkovi",           (u"alkov",         u"alkoov")),
     (u"ameba",            (u"ameb",          u"ameeb")),
     (u"arsenikki",        (u"arsenik",       u"arseniik")),
@@ -307,23 +308,37 @@ word_list = [
     (u"emali",            (u"emal",          u"emalj")),
     (u"eteinen",          (u"etei",          u"etehi")),
     (u"evankelinen",      (u"evankeli",      u"evankeeli")),
-    (u"hevonen",        (u"hevo",        u"hevoi")),
-    (u"humaaninen",     (u"humaani",     u"humani")),
-    (u"husaari",        (u"husaar",      u"husar")),
-    (u"huumori",        (u"huumor",      u"humor")),
-    (u"invalidi",       (u"invalid",     u"invaliid")),
-    (u"juridinen",      (u"juridi",      u"juriidi")),
+    (u"haupitsi",         (u"haupits",       u"haubits")),
+    (u"hevonen",          (u"hevo",          u"hevoi")),
+    (u"humaaninen",       (u"humaani",       u"humani")),
+    (u"husaari",          (u"husaar",        u"husar")),
+    (u"huumori",          (u"huumor",        u"humor")),
+    (u"inkvisiittori",    (u"inkvisiittor",  u"inkvisitor", u"inkvisiitor")),
+    (u"inkvisitio",       (u"inkvisitio",    u"inkvisiitio")),
+    (u"invalidi",         (u"invalid",       u"invaliid")),
+    (u"juridinen",        (u"juridi",        u"juriidi")),
+    (u"kahdeksainen",        (u"kahdeksai",         u"kaheksai")),
+    (u"kahdeksan",           (u"kahdeks",           u"kaheks")),
+    (u"kahdeksankymppinen",  (u"kahdeksankymppi",   u"kaheksankymppi")),
+    (u"kahdeksankertaistaa", (u"kahdeksankertaist", u"kaheksankertaist")),
+    (u"kahdeksankymmen",     (u"kahdeksankymmen",   u"kaheksankymmen")),
+    (u"kahdeksanlainen",     (u"kahdeksanlai",      u"kaheksanlai")),
+    (u"kahdeksannes",        (u"kahdeksanne",       u"kaheksanne")),
+    (u"kahdeksas",           (u"kahdeksa",          u"kaheksa")),
     (u"kaleeri",        (u"kaleer",      u"kaler")),
     (u"kamari",         (u"kamar",       u"kammar")),
     (u"kameli",         (u"kamel",       u"kameel")),
     (u"kampanja",       (u"kampanj",     u"kamppanj")),
     (u"kaneli",         (u"kanel",       u"kaneel")),
     (u"kaoottinen",     (u"kaootti",     u"kaaootti", u"kaaotti", u"kaotti")),
+    (u"kavaljeeri",     (u"kavaljeer",   u"kavalier", u"kavaljier")),
     (u"kirjoitelma",    (u"kirjoitelm",  u"kirjotelm")),
     (u"koneisto",       (u"koneisto",    u"konehisto")),
     (u"konossementti",  (u"konossement", u"konnossement")),
+    (u"kortteeri",      (u"kortteer",    u"kortter", u"kortier", u"korttier")),
     (u"kraatteri",      (u"kraatter",    u"kraater")),
     (u"kulttuuri",      (u"kulttuur",    u"kultuur", "kulttur")),
+    (u"kuriiri",        (u"kuriir",      u"kurier")),
     (u"leegio",         (u"leegio",      u"legio")),
     (u"legioona",       (u"legioon",     u"legion")),
     (u"legioonalainen", (u"legioonalai", u"legionalai")),
@@ -334,12 +349,14 @@ word_list = [
     (u"mieluinen",      (u"mielui",      u"mieluhi")),
     (u"miljoona",       (u"miljoon",     u"miljon")),
     (u"modeemi",        (u"modeem",      u"modem")),
+    (u"mosaiikki",      (u"mosaiik",     u"mosaik")),
     (u"paneeli",        (u"paneel",      u"panel")),
     (u"paronitar",      (u"paronit",     u"paroonit")),
     (u"pataljoona",     (u"pataljoon",   u"pataljon")),
     (u"patriisi",       (u"patriis",     u"patris")),
     (u"patruuna",       (u"patruun",     u"patrun")),
     (u"persoona",       (u"persoon",     u"person")),
+    (u"pioneeri",       (u"pioneer",     u"pioner")),
     (u"pioni",          (u"pion",        u"pioon")),
     (u"plutoona",       (u"plutoon",     u"pluton")),
     (u"poliisi",        (u"poliis",      u"polis")),
@@ -351,33 +368,46 @@ word_list = [
     (u"pyramidi",       (u"pyramid",     u"pyramiid")),
     (u"reettori",       (u"reettor",     u"reetor")),
     (u"sampanja",       (u"sampanj",     u"samppanj")),
+    (u"sapatti",        (u"sapat",       u"sabat")),
     (u"sapeli",         (u"sapel",       u"sapeel")),
     (u"senaatti",       ((u"senaat",     u"senaat", u"NimisanaTatti_a", u"NimisanaRisti_a"),
                          (u"senaat",     u"senat",  u"NimisanaTatti_a", u"NimisanaRisti_a"))),
     (u"senaattori",     (u"senaattor",   u"senaator")),
     (u"serafi",         (u"seraf",       u"seraaf")),
     (u"shampanja",      (u"shampanj",    u"shamppanj")),
+    (u"sihteeri",       (u"sihteer",     u"sihter", u"sihtier")),
     (u"sitruuna",       (u"sitruun",     u"sitrun", u"sitroon", u"sitron")),
     (u"soolo",          (u"soolo",       u"solo")),
-    (u"soopeli",        (u"soopel",      u"sopel")),
+    (u"soopeli",        (u"soopel",      u"sopel", u"soobel")),
     (u"synagoga",       (u"synagog",     u"synagoog")),
     (u"teatteri",       (u"teatter",     u"teaater", u"teaatter", u"teater")),
     (u"temperamentti",  (u"temperament", u"tempperament")),
     (u"tooga",          (u"toog",        u"tog")),
     (u"Toscana",        (u"toscan",      u"toskan")),
     (u"tällainen",      ((u"tällai",     u"tälläi", u"NimiLaatusanaNainen_a", u"NimiLaatusanaNainen_ä"), )),
-    (u"upseeri",        (u"upseer",      u"upser")),
+    (u"upseeri",        (u"upseer",      u"upser", u"upsier")),
+    (u"upseeristo",     (u"upseeristo",  u"upsieristo")),
+    (u"yhdeksäinen",        (u"yhdeksäi",         u"yheksäi")),
+    (u"yhdeksän",           (u"yhdeks",           u"yheks")),
+    (u"yhdeksänkymppinen",  (u"yhdeksänkymppi",   u"yheksänkymppi")),
+    (u"yhdeksänkertaistaa", (u"yhdeksänkertaist", u"yheksänkertaist")),
+    (u"yhdeksänkymmen",     (u"yhdeksänkymmen",   u"yheksänkymmen")),
+    (u"yhdeksänlainen",     (u"yhdeksänlai",      u"yheksänlai")),
+    (u"yhdeksännes",        (u"yhdeksänne",       u"yheksänne")),
+    (u"yhdeksäs",           (u"yhdeksä",          u"yheksä")),
 
     (u"ien",     [u"[Ln][Xp]ien[X]iken[Sn][Ny]e:ikene Loppu ;"]),
+##                  u"[Ln][Xp]ien[X]ien[Ses][Ny]nä:iennä NimisanaLiOlV_ä ;"
+
     (u"kappale", [u"[Ln][Xp]kappale[X]kappal[Sg][Nm]ten:kappalten # ;"]),
     (u"maailma", [u"[Ln][Xp]maailma[X]maailmoitse:maailmoitse # ;"]),
-    (u"mies",    [u"[Ln][Xp]mies[X]mies[Ses][Ny]nä:miesnä NimisanaLiOlV_ä ;",
-                  u"[Ln][Xp]mies[X]mies[Ses][Ny]sä:miessä NimisanaLiOlV_ä ;"]),
     (u"pieni",   [u"[Ll][Xp]pieni[X]pien[Ses][Ny]nä:piennä NimisanaLiOlV_ä ;"]),
     (u"sankari", [u"[Ln][Xp]sankari[X]sankar[Sg][Nm]ten:sankarten # ;"]),
     (u"tuta",    [u"[Lt][Xp]tuta[X]tu:tu SukijaTuta ;"]),
     (u"venäjä",  [u"[Ln][Xp]venäjä[X]venä[Sp][Ny]ttä:venättä NimisanaLiOlV_ä ;"]),
 
+    (u"lainen",  lambda line, word: replace_and_write (line.replace(u"lai",u"läi"), u"NimiLaatusanaNainen_a", u"NimiLaatusanaNainen_ä")),
+
     # 39 nuori (3, 3). Tuomi, s. 182, 184.
     #
     (u"juuri",   [u"[Ln][Xp]juuri[X]juur[Ses][Ny]na:juurna NimisanaLiOlV_a ;",
@@ -482,6 +512,21 @@ def convert_to_dictionary (word_list):
 sukija_dictionary = convert_to_dictionary (word_list)
 
 
+def convert_function_list_to_dictionary (function_list):
+    l0 = map (lambda x : x[1], function_list)
+    l1 = map (lambda x : [x[0] for i in range (len(x[1]))], function_list)
+    d1 = dict (zip (l0, l1))
+    d2 = dict()
+    for key, value in d1.iteritems():
+        d2.update (zip (key, value))
+    return d2
+
+
+#function_dictionary = convert_function_list_to_dictionary (function_list)
+#print (function_dictionary)
+#sukija_dictionary.update (function_dictionary)
+
+
 def error (line):
     sys.stderr.write (line)
     sys.stderr.write ("Wrong format in sukija_dictionary.\n")
@@ -512,28 +557,25 @@ def write_tuple (line, key, g):
 #
 base_form_re = re.compile (u"\\[Xp\\]([^[]+)\\[X\\]", re.UNICODE)
 
-def generate_word (line, sukija_dictionary):
-    r = base_form_re.search (line)
-    if r:
-        try:
-            g = sukija_dictionary[r.group(1)]
-	    if type(g) == ListType:
-                write_list (line, r.group(1), g)
-	    elif type(g) == TupleType:
-                write_tuple (line, r.group(1), g)
-	    else:
-		error (line)
-        except KeyError:  # It is not an error if a word is not in sukija_dictionary.
-            pass
+def generate_word (r, line, sukija_dictionary):
+    try:
+        g = sukija_dictionary[r.group(1)]
+        if type(g) == ListType:
+            write_list (line, r.group(1), g)
+	elif type(g) == TupleType:
+            write_tuple (line, r.group(1), g)
+	elif type(g) == LambdaType:
+            g (line, r.group(1))
+	else:
+            error (line)
+    except KeyError:  # It is not an error if a word is not in sukija_dictionary.
+        pass
 
 
-def generate_from_function (line, function_list):
-    r = base_form_re.search (line)
-    if r:
-        for x in function_list:
-            if r.group(1) in x[1]:
-                x[0] (line, r.group(1))
-
+def generate_from_function (r, line, function_list):
+    for x in function_list:
+        if r.group(1) in x[1]:
+            x[0] (line, r.group(1))
 
 
 # Copy Voikko vocabulary and insert forms that Sukija needs.
@@ -543,11 +585,14 @@ while True:
     if line == u"":
         break;
     outfile.write (line)
-    if line == u"LEXICON Joukahainen\n":
+    if line == u"LEXICON Sanasto\n":
         outfile.write (u"Sukija ;\n")
     generate_from_pattern (line, spelling_pattern_list)
-    generate_word (line, sukija_dictionary)
-    generate_from_function (line, function_list)
+
+    r = r = base_form_re.search (line)
+    if r:
+        generate_word (r, line, sukija_dictionary)
+        generate_from_function (r, line, function_list)
 infile.close()
 
 
